<?php

namespace Drupal\sqrl;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException;
use Drupal\Component\Plugin\Exception\PluginNotFoundException;
use Drupal\Component\Plugin\PluginBase;
use Drupal\Component\Utility\Random;
use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Entity\EntityTypeManager;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\sqrl\Entity\Identity;
use Drupal\user\Entity\User;
use Drupal\user\UserInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Base class for sqrl_action plugins.
 */
abstract class SqrlActionPluginBase extends PluginBase implements SqrlActionInterface {

  use StringTranslationTrait;

  /**
   * @var \Drupal\sqrl\Sqrl
   */
  protected $sqrl;

  /**
   * @var \Drupal\sqrl\Client
   */
  protected $client;

  /**
   * @var \Drupal\sqrl\State
   */
  protected $state;

  /**
   * @var \Drupal\sqrl\Log
   */
  protected $log;

  /**
   * @var \Drupal\Core\Entity\EntityTypeManager
   */
  protected $entityTypeManager;

  /**
   * @var \Drupal\Core\Config\ConfigFactory
   */
  protected $configFactory;

  /**
   * @var \Drupal\Component\Utility\Random
   */
  protected $random;

  /**
   * @var \Drupal\user\Entity\User
   */
  protected $user;

  /**
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected $time;

  /**
   * @var \Drupal\sqrl\SqrlActionPluginManager
   */
  protected $actionPluginManager;

  /**
   * @var \Drupal\sqrl\Identities
   */
  protected $identities;

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, Sqrl $sqrl, Client $client, State $state, Log $log, EntityTypeManager $entity_type_manager, ConfigFactory $config_factory, TimeInterface $time, SqrlActionPluginManager $sqrl_action_plugin_manager, Identities $identities) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->sqrl = $sqrl;
    $this->client = $client;
    $this->state = $state;
    $this->log = $log;
    $this->entityTypeManager = $entity_type_manager;
    $this->configFactory = $config_factory;
    $this->time = $time;
    $this->actionPluginManager = $sqrl_action_plugin_manager;
    $this->identities = $identities;

    $this->random = new Random();
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('sqrl.handler'),
      $container->get('sqrl.client'),
      $container->get('sqrl.state'),
      $container->get('sqrl.log'),
      $container->get('entity_type.manager'),
      $container->get('config.factory'),
      $container->get('datetime.time'),
      $container->get('plugin.manager.sqrl_action'),
      $container->get('sqrl.identities')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function label(): string {
    return (string) $this->pluginDefinition['label'];
  }

  /**
   * {@inheritdoc}
   */
  public function requiresSignatureRevalidation(): bool {
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function setUser(UserInterface $user = NULL): SqrlActionInterface {
    if (($user === NULL) && $uid = $this->sqrl->getNut()->getClientUid()) {
      /** @var \Drupal\user\UserInterface $user */
      $this->user = User::load($uid);
    }
    else {
      $this->user = $user;
    }
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function rememberSuccess(): SqrlActionInterface {
    if ($this->user) {
      $this->state->setAuth($this->client->getNut()->getPublicNut(), $this->user->id());
    }
    return $this;
  }

}
